<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2022 Ravi Trébuchet <ravi@code42.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * \file    produitrapide/admin/setup.php
 * \ingroup produitrapide
 * \brief   ProduitRapide setup page.
 */

// Load Dolibarr environment
$res = 0;
// Try main.inc.php into web root known defined into CONTEXT_DOCUMENT_ROOT (not always defined)
if (!$res && !empty($_SERVER["CONTEXT_DOCUMENT_ROOT"])) {
	$res = @include $_SERVER["CONTEXT_DOCUMENT_ROOT"] . "/main.inc.php";
}
// Try main.inc.php into web root detected using web root calculated from SCRIPT_FILENAME
$tmp = empty($_SERVER['SCRIPT_FILENAME']) ? '' : $_SERVER['SCRIPT_FILENAME'];
$tmp2 = realpath(__FILE__);
$i = strlen($tmp) - 1;
$j = strlen($tmp2) - 1;
while ($i > 0 && $j > 0 && isset($tmp[$i]) && isset($tmp2[$j]) && $tmp[$i] == $tmp2[$j]) {
	$i--;
	$j--;
}
if (!$res && $i > 0 && file_exists(substr($tmp, 0, ($i + 1)) . "/main.inc.php")) {
	$res = @include substr($tmp, 0, ($i + 1)) . "/main.inc.php";
}
if (!$res && $i > 0 && file_exists(dirname(substr($tmp, 0, ($i + 1))) . "/main.inc.php")) {
	$res = @include dirname(substr($tmp, 0, ($i + 1))) . "/main.inc.php";
}
// Try main.inc.php using relative path
if (!$res && file_exists("../../main.inc.php")) {
	$res = @include "../../main.inc.php";
}
if (!$res && file_exists("../../../main.inc.php")) {
	$res = @include "../../../main.inc.php";
}
if (!$res) {
	die("Include of main fails");
}

global $langs, $db, $user, $conf, $hookmanager, $mysoc;

// Libraries
require_once DOL_DOCUMENT_ROOT . "/core/lib/admin.lib.php";
dol_include_once('/produitrapide/lib/produitrapide.lib.php');

// Translations
$langs->loadLangs(array("admin", "produitrapide@produitrapide"));

// Initialize technical object to manage hooks of page. Note that conf->hooks_modules contains array of hook context
$hookmanager->initHooks(array('produitrapidesetup', 'globalsetup'));

// Access control
if (!$user->admin) {
	accessforbidden();
}

// Parameters
$action = GETPOST('action', 'aZ09');
$backtopage = GETPOST('backtopage', 'alpha');
$modulepart = GETPOST('modulepart', 'aZ09');    // Used by actions_setmoduleoptions.inc.php

$value = GETPOST('value', 'alpha');
$label = GETPOST('label', 'alpha');
$scandir = GETPOST('scan_dir', 'alpha');
$type = 'myobject';

$error = 0;
$setupnotempty = 0;

// Set this to 1 to use the factory to manage constants. Warning, the generated module will be compatible with version v15+ only
$useFormSetup = 1;

if (!class_exists('FormSetup')) {
	// For retrocompatibility Dolibarr < 16.0
	if (floatval(DOL_VERSION) < 16.0 && !class_exists('FormSetup')) {
		require_once __DIR__ . '/../backport/v16/core/class/html.formsetup.class.php';
	} else {
		require_once DOL_DOCUMENT_ROOT . '/core/class/html.formsetup.class.php';
	}
}

$formSetup = new FormSetup($db);

$form = new Form($db);

$item = $formSetup->newItem('PRODUITRAPIDE_DEFAULT_CREATE');
$itemFields = array(
	'0' => $langs->trans('Product'),
	'1' => $langs->trans('Service')
);
$item->setAsSelect($itemFields);

// [#1] Wysiwyg field doesn't work on fancybox popup
$formSetup->newItem('PRODUITRAPIDE_DISPLAY_DESCRIPTION')->setAsYesNo();
$formSetup->newItem('PRODUITRAPIDE_DISPLAY_URL')->setAsYesNo();
$formSetup->newItem('PRODUITRAPIDE_DISPLAY_PRICE')->setAsYesNo();
$item = $formSetup->newItem('PRODUITRAPIDE_TVA');
$form->load_cache_vatrates("'" . $mysoc->country_code . "'"); // If no vat defined, return -1 with message into this->error
$itemFields = array();
foreach ($form->cache_vatrates as $rate) {
	$key = $rate['txtva'];
	$key .= $rate['nprtva'] ? '*' : '';
	$key .= ' ('.$rate['code'].')';
	$itemFields[$key] = $rate['label'];
}

$item->setAsSelect($itemFields);
$formSetup->newItem('PRODUITRAPIDE_DISPLAY_FILES')->setAsYesNo();
$formSetup->newItem('PRODUITRAPIDE_DISPLAY_SUPPLIER')->setAsYesNo();

if ($conf->barcode->enabled ?? false) {
	$formSetup->newItem('PRODUITRAPIDE_DISPLAY_BARCODE')->setAsYesNo();
}
$formSetup->newItem('PRODUITRAPIDE_ENABLE_BUTTON_PROPAL')->setAsYesNo();
if ($conf->facture->enabled ?? false) {
	$formSetup->newItem('PRODUITRAPIDE_ENABLE_BUTTON_INVOICE')->setAsYesNo();
}
if ($conf->commande->enabled ?? false) {
	$formSetup->newItem('PRODUITRAPIDE_ENABLE_BUTTON_ORDER')->setAsYesNo();
}
if ($conf->fournisseur->enabled ?? false) {
	$formSetup->newItem('PRODUITRAPIDE_ENABLE_BUTTON_ORDER_SUPPLIER')->setAsYesNo();
}
$formSetup->newItem('PRODUIT_RAPIDE_LINE_COLOR')->setAsColor();

$setupnotempty += count($formSetup->items);

/*
 * Actions
 */

// For retrocompatibility Dolibarr < 15.0
if (versioncompare(explode('.', DOL_VERSION), array(15)) < 0 && $action == 'update' && !empty($user->admin)) {
	$formSetup->saveConfFromPost();
}

include DOL_DOCUMENT_ROOT . '/core/actions_setmoduleoptions.inc.php';

/*
 * View
 */

$form = new Form($db);

$help_url = '';
$page_name = "ProduitRapideSetup";

$jsArray = array(dol_buildpath("/produitrapide/js/setup.js.php", 1));

llxHeader('', $langs->trans($page_name), $help_url, '', 0, 0, $jsArray);

// Subheader
$linkback = '<a href="' . ($backtopage ? $backtopage : DOL_URL_ROOT . '/admin/modules.php?restore_lastsearch_values=1') . '">' . $langs->trans("BackToModuleList") . '</a>';

print load_fiche_titre($langs->trans($page_name), $linkback, 'title_setup');

// Configuration header
$head = produitrapideAdminPrepareHead();
print dol_get_fiche_head($head, 'settings', $langs->trans($page_name), -1, "produitrapide@produitrapide");

// Setup page goes here
echo '<span class="opacitymedium">' . $langs->trans("ProduitRapideSetupPage") . '</span><br><br>';

if ($action == 'edit') {
	print $formSetup->generateOutput(true);
	print '<br>';
} elseif (!empty($formSetup->items)) {
	print $formSetup->generateOutput();
	print '<div class="tabsAction">';
	print '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?action=edit&token=' . newToken() . '">' . $langs->trans("Modify") . '</a>';
	print '</div>';
} else {
	print '<br>' . $langs->trans("NothingToSetup");
}

// Page end
print dol_get_fiche_end();

llxFooter();
$db->close();
