<?php
/* Copyright (C) 2022 Ravi Trébuchet <ravi@code42.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * \file    produitrapide/class/actions_produitrapide.class.php
 * \ingroup produitrapide
 * \brief   Example hook overload.
 *
 * Put detailed description here.
 */

dol_include_once('/produitrapide/lib/produitrapide.lib.php');

/**
 * Class ActionsProduitRapide
 */
class ActionsProduitRapide
{
	/**
	 * @var DoliDB Database handler.
	 */
	public $db;

	/**
	 * @var string Error code (or message)
	 */
	public $error = '';

	/**
	 * @var array Errors
	 */
	public $errors = array();


	/**
	 * @var array Hook results. Propagated to $hookmanager->resArray for later reuse
	 */
	public $results = array();

	/**
	 * @var string String displayed by executeHook() immediately after return
	 */
	public $resprints;

	/**
	 * @var int		Priority of hook (50 is used if value is not defined)
	 */
	public $priority;


	/**
	 * Constructor
	 *
	 *  @param		DoliDB		$db      Database handler
	 */
	public function __construct($db)
	{
		$this->db = $db;
	}


	/**
	 * Execute action
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function getNomUrl($parameters, &$object, &$action)
	{
		global $db, $langs, $conf, $user;
		$this->resprints = '';
		return 0;
	}

	/**
	 * Overloading the doActions function : replacing the parent's function with the one below
	 *
	 * @param   array           $parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          $action         Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Returns < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function doActions($parameters, &$object, &$action, $hookmanager)
	{
		dol_include_once('/produitrapide/lib/produitrapide.lib.php');
		global $user, $db, $langs, $conf;

		$error = 0; // Error counter
		$langs->load("produitrapide@produitrapide");

		// Fast Product
		if (in_array($parameters['currentcontext'], array('propalcard', 'invoicecard', 'ordercard', 'ordersuppliercard'))) {
			if ($action == "createfastproduct") {
				if (($user->rights->produit->creer ?? false) || ($user->rights->service->creer ?? false)) {
					dol_include_once('/produitrapide/class/fastproduct.class.php');
					$product = new FastProduct($db);
					if ($product->createFastProduct($object) > 0) {
						exit(header("Location: " . $_SERVER['PHP_SELF'] . "?id=" . $object->id));
					} else {
						$action = 'fastproduct';
					}
				}
			} else if ($action == 'fastproduct') {
				if (!generateFastProductWindow($object)) {
					$error++;
				}
			}
		}

		if (!$error) {
			$this->results = array('myreturn' => 999);
			$this->resprints = $langs->trans('ProduitRapideCreationSuccess');
			return 0;
		} else {
			$this->errors[] = $langs->trans('ProduitRapideCreationFailed');
			return -1;
		}
	}


	/**
	 * Overloading the doMassActions function : replacing the parent's function with the one below
	 *
	 * @param   array           $parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          $action         Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Returns < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function doMassActions($parameters, &$object, &$action, $hookmanager)
	{
		return 0;
	}

	/**
	 * Adds more actions buttons.
	 *
	 * @param   array           $parameters    Hook metadatas (context, etc...)
	 * @param   CommonObject    $object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...
	 * @param   string          $action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager   Hook manager propagated to allow calling another hook
	 * @return  int                            Returns < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function addMoreActionsButtons($parameters, &$object, &$action, $hookmanager): int
	{
		global $langs, $conf;

		dol_include_once('/produitrapide/class/fastproduct.class.php');

		$langs->load("produitrapide@produitrapide");
		dol_syslog('produitrapide - addMoreActionsButtons activation', LOG_DEBUG);

		// [#29] We hide fast product button if we are in edit mode
		if ($action != 'editline') {
			if (in_array('propalcard', explode(':', $parameters['context']))) {
				if (isset($conf->global->PRODUITRAPIDE_ENABLE_BUTTON_PROPAL) && $conf->global->PRODUITRAPIDE_ENABLE_BUTTON_PROPAL && $object->status == 0) {
					buttonContextGlobalCard($object, $langs);
				}
			} else if (in_array('invoicecard', explode(':', $parameters['context']))) {
				if (isset($conf->global->PRODUITRAPIDE_ENABLE_BUTTON_INVOICE) && $conf->global->PRODUITRAPIDE_ENABLE_BUTTON_INVOICE && $object->status == 0) {
					buttonContextGlobalCard($object, $langs);
				}
			} else if (in_array('ordercard', explode(':', $parameters['context']))) {
				if (isset($conf->global->PRODUITRAPIDE_ENABLE_BUTTON_ORDER) && $conf->global->PRODUITRAPIDE_ENABLE_BUTTON_ORDER && $object->status == 0) {
					buttonContextGlobalCard($object, $langs);
				}
			} else if (in_array('ordersuppliercard', explode(':', $parameters['context']))) {
				if (isset($conf->global->PRODUITRAPIDE_ENABLE_BUTTON_ORDER_SUPPLIER) && $conf->global->PRODUITRAPIDE_ENABLE_BUTTON_ORDER_SUPPLIER && ($conf->global->PRODUITRAPIDE_DISPLAY_SUPPLIER ?? false) && $object->status == 0) {
					buttonContextGlobalCard($object, $langs);
				}
			}
		}

		return 0;
	}

	/**
	 * Execute action completeTabsHead
	 *
	 * @param   array           $parameters     Array of parameters
	 * @param   CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          $action         'add', 'update', 'view'
	 * @param   Hookmanager     $hookmanager    hookmanager
	 * @return  int                             <0 if KO,
	 *                                          =0 if OK but we want to process standard actions too,
	 *                                          >0 if OK and we want to replace standard actions.
	 */
	public function completeTabsHead(&$parameters, &$object, &$action, $hookmanager)
	{
		global $langs, $conf, $user;

		if (!isset($parameters['object']->element)) {
			return 0;
		}
		if ($parameters['mode'] == 'remove') {
			// utilisé si on veut faire disparaitre des onglets.
			return 0;
		} elseif ($parameters['mode'] == 'add') {
			$langs->load('produitrapide@produitrapide');
			// utilisé si on veut ajouter des onglets.
			$counter = count($parameters['head']);
			$element = $parameters['object']->element;
			$id = $parameters['object']->id;
			// verifier le type d'onglet comme member_stats où ça ne doit pas apparaitre
			// if (in_array($element, ['societe', 'member', 'contrat', 'fichinter', 'project', 'propal', 'commande', 'facture', 'order_supplier', 'invoice_supplier'])) {
			if (in_array($element, ['context1', 'context2'])) {
				$datacount = 0;

				$parameters['head'][$counter][0] = dol_buildpath('/produitrapide/produitrapide_tab.php', 1) . '?id=' . $id . '&amp;module='.$element;
				$parameters['head'][$counter][1] = $langs->trans('ProduitRapideTab');
				if ($datacount > 0) {
					$parameters['head'][$counter][1] .= '<span class="badge marginleftonlyshort">' . $datacount . '</span>';
				}
				$parameters['head'][$counter][2] = 'produitrapideemails';
				$counter++;
			}
			if ($counter > 0 && (int) DOL_VERSION < 14) {
				$this->results = $parameters['head'];
				// return 1 to replace standard code
				return 1;
			} else {
				// en V14 et + $parameters['head'] est modifiable par référence
				return 0;
			}
		}
	}


	/**
	 * Get the module header for information tab
	 *
	 * @param array $parameters Array of parameter (we only use $parameters['module'] to check module header to display)
	 * @return  int                             < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function generateInformationTabHeader($parameters)
	{
		dol_include_once('/produitrapide/lib/produitrapide.lib.php');

		// Check if we need to display contrat plus information header
		if ($parameters['module'] == 'modProduitRapide') {
			$this->results['head'] = produitrapideAdminPrepareHead();
			$this->results['active_tab'] = 'information';
			$this->results['langs'] = 'produitrapide@produitrapide';
			$this->results['header_icon'] = 'produitrapide@produitrapide';
			return 1;
		}
	}

	/**
	 * Get the module header for information tab
	 *
	 * @param array $parameters Array of parameter (we only use $parameters['module'] to check module header to display)
	 * @return  int                             < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function generateMigrationTabHeader($parameters)
	{
		dol_include_once('/produitrapide/lib/produitrapide.lib.php');

		// Check if we need to display contrat plus information header
		if ($parameters['module'] == 'modProduitRapide') {
			$this->results['head'] = produitrapideAdminPrepareHead();
			$this->results['active_tab'] = 'migration';
			$this->results['langs'] = 'produitrapide@produitrapide';
			$this->results['header_icon'] = 'produitrapide@produitrapide';
			return 1;
		}
	}

	/**
	 * Execute action menuDropdownQuickaddItems
	 *
	 * @param   array           $parameters     Array of parameters
	 * @param   CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          $action         'add', 'update', 'view'
	 * @param   Hookmanager     $hookmanager    hookmanager
	 * @return  int                             <0 if KO,
	 *                                          =0 if OK but we want to process standard actions too,
	 *                                          >0 if OK and we want to replace standard actions.
	 */
	public function menuDropdownQuickaddItems(&$parameters, &$object, &$action, $hookmanager)
	{
		global $user;

		$hookmanager->results['items'] = array(
			"url" => dol_buildpath('/custom/produitrapide/createfastproduct.php?action=create&idmenu=340&mainmenu=products&leftmenu=', 1),
			"title" => "ProduitRapideAddProduct@produitrapide",
			"name" => "ProduitRapideQuickAdd@produitrapide",
			"picto" => "product",
			"activation" => ($user->rights->produit->creer ?? false),
			"position" => 505,
		);

		return 0;
	}

	/**
	 * Execute action menuDropdownQuickaddItems
	 *
	 * @param   array           $parameters     Array of parameters
	 * @param   CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          $action         'add', 'update', 'view'
	 * @param   Hookmanager     $hookmanager    hookmanager
	 * @return  int                             <0 if KO,
	 *                                          =0 if OK but we want to process standard actions too,
	 *                                          >0 if OK and we want to replace standard actions.
	 */
	public function printObjectLine(&$parameters, &$object, &$action, $hookmanager)
	{
		global $conf;

		if (isFastProductLine($parameters["line"])) {
			$color = $conf->global->PRODUIT_RAPIDE_LINE_COLOR ?? "#FFFFE0";
			print "<style> #row-" . $parameters["line"]->id . " { background: " . $color . "; } </style>";
		}

		return 0;
	}
}
