<?php
/* Copyright (C) 2022 Ravi TREBUCHET <ravi@code42.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Library javascript to enable Browser notifications
 */

//if (!defined('NOREQUIREUSER')) define('NOREQUIREUSER', '1');
//if (!defined('NOREQUIRESOC')) define('NOREQUIRESOC', '1');
//if (!defined('NOCSRFCHECK')) define('NOCSRFCHECK', 1);
//if (!defined('NOTOKENRENEWAL')) define('NOTOKENRENEWAL', 1);
//if (!defined('NOLOGIN')) define('NOLOGIN', 1);
//if (!defined('NOREQUIREMENU')) define('NOREQUIREMENU', 1);
//if (!defined('NOREQUIREHTML')) define('NOREQUIREHTML', 1);
//if (!defined('NOREQUIREAJAX')) define('NOREQUIREAJAX', '1');

/**
 * \file    produitrapide/js/default_list.js.php
 * \ingroup produitrapide
 * \brief   Generic javaScript file to manage dolibarr listing.
 */

// Load Dolibarr environment
$res = 0;
// Try main.inc.php into web root known defined into CONTEXT_DOCUMENT_ROOT (not always defined)
if (!$res && !empty($_SERVER["CONTEXT_DOCUMENT_ROOT"])) $res = @include $_SERVER["CONTEXT_DOCUMENT_ROOT"] . "/main.inc.php";
// Try main.inc.php into web root detected using web root calculated from SCRIPT_FILENAME
$tmp = empty($_SERVER['SCRIPT_FILENAME']) ? '' : $_SERVER['SCRIPT_FILENAME'];$tmp2 = realpath(__FILE__); $i = strlen($tmp) - 1; $j = strlen($tmp2) - 1;
while ($i > 0 && $j > 0 && isset($tmp[$i]) && isset($tmp2[$j]) && $tmp[$i] == $tmp2[$j]) {
	$i--;
	$j--;
}
if (!$res && $i > 0 && file_exists(substr($tmp, 0, ($i + 1)) . "/main.inc.php")) $res = @include substr($tmp, 0, ($i + 1)) . "/main.inc.php";
if (!$res && $i > 0 && file_exists(substr($tmp, 0, ($i + 1)) . "/../main.inc.php")) $res = @include substr($tmp, 0, ($i + 1)) . "/../main.inc.php";
// Try main.inc.php using relative path
if (!$res && file_exists("../../main.inc.php")) $res = @include "../../main.inc.php";
if (!$res && file_exists("../../../main.inc.php")) $res = @include "../../../main.inc.php";
if (!$res) die("Include of main fails");

// Define js type
header('Content-Type: application/javascript');
// Important: Following code is to cache this file to avoid page request by browser at each Dolibarr page access.
// You can use CTRL+F5 to refresh your browser cache.
if (empty($dolibarr_nocache)) header('Cache-Control: max-age=3600, public, must-revalidate');
else header('Cache-Control: no-cache');

global $langs;

$langs->load('produitrapide@produitrapide');

?>

/**
 * Inputs corresponds to fields to check
 * id is the id of input
 * warning is nessessary to display only one warning at time
 * voidValue is the void value of field
 * required : true if it is a required field
 *            or different elements set into inputs to check values of "required" if field is filled (if field is filled, form will be sent only if all fields set into required are filled too)
 *
 * @type {{fp_fk_supplier: {voidValue: string, warning: boolean, id: string, required: {fp_supplierref: string, fp_supplierprice: string}}, fp_label: {voidValue: string, warning: boolean, id: string, required: boolean}, fp_supplierref: {voidValue: string, warning: boolean, id: string, required: {fp_fk_supplier: string, fp_supplierprice: string}}, fp_supplierprice: {voidValue: string, warning: boolean, id: string, required: {fp_fk_supplier: string, fp_supplierref: string}}, fp_ref: {voidValue: string, warning: boolean, id: string, required: boolean}}}
 */
let inputs = {
	fp_ref: {
		id: "fp_ref",
		warning: false,
		voidValue: "",
		required: true
	},
	fp_label: {
		id: "fp_label",
		warning: false,
		voidValue: "",
		required: true
	},
	fp_fk_supplier: {
		id: "fp_fk_supplier",
		warning: false,
		voidValue: "-1",
		required: {
			fp_supplierref: "",
			fp_supplierprice: ""
		}
	},
	fp_supplierref: {
		id: "fp_supplierref",
		warning: false,
		voidValue: "",
		required: {
			fp_fk_supplier: "",
			fp_supplierprice: ""
		}
	},
	fp_supplierprice: {
		id: "fp_supplierprice",
		warning: false,
		voidValue: "",
		required: {
			fp_fk_supplier: "",
			fp_supplierref: ""
		}
	}
}
/**
 * Launched when page is loaded
 */
$(document).ready(function () {

	// [#21] If object is supplier order, supplier information are required
	if (window.location.href.includes('/fourn/commande/card.php')) {
		inputs['fp_fk_supplier']['required'] = true;
		inputs['fp_supplierref']['required'] = true;
		inputs['fp_supplierprice']['required'] = true;
	}

	// Add input events to delete warnings if something is typed
	for (var field in inputs) {
		let input = document.getElementById(inputs[field]["id"])
		if (input) {
			input.addEventListener('input', deleteWarningOnInput);
		}
	}

	// On supplier selection, test if we have to delete warning or not
	$("#fp_fk_supplier").change(function() {
		testSupplierField()
	});

	// On supplier search, test if we have to delete warning or not
	$("#search_fp_fk_supplier").change(function() {
		testSupplierField()
	});

	// On type selection, change product default reference
	$("#fp_type").change(function() {
		replaceDefaultRef();
	});

	// On supplier selection, change vat rate in function of supplier country
	$("#fp_fk_supplier").change(function() {
		replaceSupplierTvaRate();
	});

	// Test if fields aren't empty on submit
	$("a#submitButtonFP").click(() => {
		// Submit form only if ref and label are completed AND if supplier is well completed
		if (testRequiredFields(true)) {
			document.fastProductForm.submit();
		}
	});
})

/**
 * Test fields before sending form
 *
 * @param isSubmit
 * @returns {boolean}
 */
function testRequiredFields(isSubmit) {

	// Check Ref and Label fields
	testOk = testFields(isSubmit);

	// Check supplier fields
	if (testSupplier(isSubmit) && testOk) {
		testOk = true;
	} else {
		testOk = false;
	}

	return testOk;
}

/**
 * Create "Warning" element
 *
 * @param id The id of field
 * @returns {HTMLSpanElement} Warning element
 */
function createWarning(id) {
	let warning = document.createElement('span')
	warning.setAttribute('id', id)
	warning.innerHTML = '<span class="fa fa-warning"></span> <?php echo $langs->trans('ProduitRapideFieldRequired') ?>'
	warning.setAttribute('style', 'color: red; float: right;')
	return warning
}

/**
 * Set warning element next to the right field
 *
 * @param element The name of element in which to put warning
 */
function setWarning(element) {
	if (inputs[element]["warning"] == false) {
		let warn = createWarning(inputs[element]["id"] + "_warning")
		let elem = document.getElementById(inputs[element]["id"])
		elem.parentElement.appendChild(warn);
		inputs[element]["warning"] = true;
	}
}

/**
 * Delete warning element if exists
 *
 * @param element The name of field
 */
function deleteWarning(element) {
	let warn = document.getElementById(inputs[element]["id"] + "_warning")
	if (warn !== null) {
		warn.remove()
	}
	inputs[element]["warning"] = false;
}
/**
 * Delete warning element if exists on input event
 *
 * @param e The name of field
 */
function deleteWarningOnInput(e) {
	let warn = document.getElementById(e.target.id + "_warning")
	if (warn !== null) {
		warn.remove()
	}
	inputs[e.target.id]["warning"] = false;
}

/**
 * Check if required fields are filled
 *
 * @param isSubmit
 * @returns {boolean} True if all required fields are filled and False if not
 */
function testFields(isSubmit) {
	let testOk = true;
	for (var input in inputs) {
		// If field is required, add warning if it's not filled, or we delete warning
		if (inputs[input]["required"] === true) {
			let elem = document.getElementById(inputs[input]["id"]);
			if (elem && (elem.value === inputs[input]["voidValue"] || elem.value === '')) {
				testOk = false;
				if (!inputs[input]["warning"] && isSubmit) {
					setWarning(input)
				}
			} else {
				deleteWarning(input)
			}
		}
	}

	return testOk;
}

/**
 * Check if non required fields are filled
 *
 * @param isSubmit
 * @returns {boolean} True if all non required fields are filled and False if not
 */
function testSupplier(isSubmit) {
	let testOk = true;

	for (var input in inputs) {
		if (inputs[input]["required"] !== true) {
			let elem = document.getElementById(inputs[input]["id"]);
			// If field is not required but require other fields, we add warning in these other fields
			if (elem && elem.value !== inputs[input]["voidValue"] && elem.value !== "" && elem.value !== null) {
				let testSup = testSupplierInput(isSubmit, input)
				if (!testSup || (testOk && inputs[input]["warning"] === true)) {
					testOk = false;
				}
			}
			// Delete warning if fields are okay
			if (inputs[input]["warning"] && testOk) {
				deleteWarning(input)
			}
		}
	}

	return testOk;
}

/**
 * For a non required field, check if its required fields are filled
 *
 * @param isSubmit
 * @param input Name of field to test
 */
function testSupplierInput(isSubmit, input) {
	let testOk = true;
	for (var requiredField in inputs[input]["required"]) {
		let voidValue = inputs[requiredField]["voidValue"]
		let elem = document.getElementById(inputs[requiredField]["id"]);
		if (elem && (elem.value === voidValue || elem.value === "")) {
			testOk = false
			if (!inputs[requiredField]["warning"] && isSubmit) {
				setWarning(requiredField)
			}
		} else {
			deleteWarning(requiredField)
		}

	}

	return testOk
}

/**
 * Test if value is typed into supplier and delete warning if value isn't empty
 */
function testSupplierField() {
	elem = document.getElementById('fp_fk_supplier');
	elemId = "fp_fk_supplier";
	if (elem && elem.value != inputs[elemId]['voidValue'] && elem.value != '') {
		deleteWarning(elemId)
	} else {
		setWarning(elemId)
	}
}

/**
 * Update default reference in function of the selected object type
 *
 */
function replaceDefaultRef() {

	let ref = document.getElementById('fp_ref');
	let type = document.getElementById('fp_type');

	var url = "<?php echo dol_buildpath('/produitrapide/ajax/ajax_getDefaultRef.php', 1)?>";

	// Ajax to replace product/service reference if default value of reference is set
	$.ajax({
		url: url,
		type: "GET",
		data : {
			token: "<?php echo newToken(); ?>",
			type: type.value
		},
		success: function (result) {
			result = JSON.parse(result);
			if (result.success) {
				if (result.value && ref) {
					ref.value = result.value;
				}
			}
		}
	});
}
/**
 * Replace supplier vat rate in function of selected supplier
 */
function replaceSupplierTvaRate() {

	let vat = document.getElementById('fourn_tva');
	let supplier = document.getElementById('fp_fk_supplier');
	let vatSelect = document.getElementById('fp_vat_rate_supplier');

	if (vatSelect !== null) {
		var selectedVat = vatSelect.selectedOptions[0].value;

		var url = "<?php echo dol_buildpath('/produitrapide/ajax/ajax_get_tva_rate.php', 1)?>";

		// Ajax to replace product/service reference if default value of reference is set
		$.ajax({
			url: url,
			type: "GET",
			data: {
				token: "<?php echo newToken();?>",
				supplier: supplier.value,
				selected_vat: selectedVat
			},
			success: function (result) {
				result = JSON.parse(result);
				if (result.success) {
					if (result.value && vat) {
						vat.innerHTML = result.value;
					}
				}
			}
		});
	}
}

/**
 * Function to allow user
 *
 * @param e
 */
function preventNonNumericalInput(e) {
	e = e || window.event;
	var charCode = (typeof e.which == "undefined") ? e.keyCode : e.which;
	var charStr = String.fromCharCode(charCode);

	if (!charStr.match(/^[0-9,.]+$/))
		e.preventDefault();
}
