<?php
/* Copyright (C) 2022 Ravi Trébuchet <ravi@code42.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * \file    produitrapide/lib/produitrapide.lib.php
 * \ingroup produitrapide
 * \brief   Library files with common functions for ProduitRapide
 */

/**
 * Prepare admin pages header
 *
 * @return array
 */
function produitrapideAdminPrepareHead()
{
	global $langs, $conf, $user;

	$langs->load("produitrapide@produitrapide");

	$h = 0;
	$head = array();

	$head[$h][0] = dol_buildpath("/produitrapide/admin/setupfastproduct.php", 1);
	$head[$h][1] = '<i class="fas fa-cog"></i> ' . $langs->trans("ProduitRapideSettingsFastProduct");
	$head[$h][2] = 'settings';
	$h++;

	$head[$h][0] = dol_buildpath("/produitrapide/admin/user_doc.php", 1);
	$head[$h][1] = '<i class="fas fa-user"></i> ' . $langs->trans("ProduitRapideUserDoc");
	$head[$h][2] = 'user_doc';
	$h++;

	$head[$h][0] = dol_buildpath("/produitrapide/admin/changelog.php", 1);
	$head[$h][1] = '<i class="fas fa-exclamation"></i> ' . $langs->trans("ProduitRapideChangelog");
	$head[$h][2] = 'changelog';
	$h++;

	if ($conf->h2g2->enabled) {
		dol_include_once('/h2g2/lib/h2g2.lib.php');
		$langs->load('h2g2@h2g2');

		if ($user->admin && isH2G2InstalledWithMinVersion('15.0.03')) {
			// H2G2 migration page only for admin
			$head[$h][0] = dol_buildpath('/h2g2/admin/migration_page.php?module=modProduitRapide&modulePath=/produitrapide/core/modules/modProduitRapide.class.php', 1);
			$head[$h][1] = '<i class="fas fa-wrench"></i> ' . $langs->trans("ProduitRapideMigrationPageTitle");
			$head[$h][2] = 'migration';
			$h++;

			// H2G2 information page
			$head[$h][0] = dol_buildpath('/h2g2/admin/information_page.php?module=modProduitRapide&modulePath=/produitrapide/core/modules/modProduitRapide.class.php', 1);
			$head[$h][1] = '<i class="fas fa-info-circle"></i> ' . $langs->trans("ProduitRapideInformationPageTitle");
			$head[$h][2] = 'information';
			$h++;
		}
	}

	complete_head_from_modules($conf, $langs, null, $head, $h, 'produitrapide@produitrapide');

	complete_head_from_modules($conf, $langs, null, $head, $h, 'produitrapide@produitrapide', 'remove');

	return $head;
}

/**
 *
 * @param string $filename File to open
 * @param bool $isFullPath True if file is in module source folder
 * @return  bool|null|string                Content or null
 */
function getProduitRapideDocumentationContent($filename, $isFullPath = true)
{
	require_once DOL_DOCUMENT_ROOT . '/core/lib/files.lib.php';

	$content = null;

	$pathoffile = dol_buildpath('/produitrapide/' . $filename, 0);
	if (dol_is_file($pathoffile)) {     // Mostly for external modules
		// str_replace is necessary if
		$content = file_get_contents($pathoffile);
		if (!$isFullPath) $content = str_replace('(./img/', '(../img/', $content);
		require_once DOL_DOCUMENT_ROOT . '/includes/parsedown/Parsedown.php';
		$parsedown = new Parsedown();
		$parsedown->setSafeMode(false); // This will escape HTML link <a href=""> into html entities but markdown links are ok
		$content = $parsedown->text($content);
	}
	return $content;
}

/**
 * @param CommonObject|null $object The current objecrt
 * @param bool $display True if we show form and False if not
 * @return string
 */
function generateFastProductWindow(CommonObject $object = null, bool $display = false): string
{
	global $db, $langs, $conf, $user, $mysoc;

	$langs->load('produitrapide@produitrapide');

	require_once DOL_DOCUMENT_ROOT . '/core/class/html.form.class.php';
	require_once DOL_DOCUMENT_ROOT . '/product/stock/class/entrepot.class.php';
	dol_include_once('/produitrapide/class/fastproduct.class.php');

	$product = new FastProduct($db);

	$htmlToPrint = '<script type="text/javascript" src=' . dol_buildpath("/produitrapide/js/jquery.fancybox.min.js", 1) . '></script>';
	$htmlToPrint .= '<script type="text/javascript" src=' . dol_buildpath("/produitrapide/js/produitrapidecreate.js.php", 1) . '></script>';
	$htmlToPrint .= '<link rel="stylesheet" href="' . dol_buildpath("/produitrapide/css/jquery.fancybox.min.css", 1) . '">';
	$htmlToPrint .= '<link rel="stylesheet" href="' . dol_buildpath("/produitrapide/css/produitrapidecreate.css", 1) . '">';
	$htmlToPrint .= '<div  id="fastproduct" class="pr-fancybox" style="display:' . ($display ? '' : 'none') . '">';
	// Prints Title
	$htmlToPrint .= '<div class="titre' . ($display ? '' : ' center') .'">';
	$htmlToPrint .= '<span class="fas fa-cube valignmiddle widthpictotitle pictotitle" style="color: #a69944;"></span> ' . $langs->trans('ProduitRapideFastProductTitle');
	$htmlToPrint .= '</div>';

	$htmlToPrint .=  dol_get_fiche_head(array(), '');
	// Prints form
	$htmlToPrint .= '<form name="fastProductForm" action="' . $_SERVER['PHP_SELF'] . (!empty($object->id) ? '?id=' . $object->id : "") . '" method="post" enctype="multipart/form-data">';
	$htmlToPrint .= '<input type="hidden" name="action" value="createfastproduct">';
	$htmlToPrint .= '<input type="hidden" name="token" value="' . newToken() . '">';
	$htmlToPrint .= '<table class="border centpercent tableforfieldedit">';
	$htmlToPrint .= '<tbody>';

	$form = new Form($db);
	// Foreach FastProduct field, we print field title and input if it is enabled and visible
	foreach ($product->fields as $fieldKey => $field) {
		// If field enabled
		if (array_key_exists('enabled', $field) && isset($field['enabled']) && !verifCond($field['enabled'])) {
			continue; // We don't want this field
		}
		// If field is visible
		if (array_key_exists('visible', $field) && isset($field['visible']) && abs($field['visible']) != 1) {
			continue;
		}
		// Set order of product type depending on configuration
		if ($fieldKey == "fp_type") {
			$productConfig = (int) $conf->global->PRODUITRAPIDE_DEFAULT_CREATE ?? 0;
			if ($productConfig == 0) {
				if ($user->rights->produit->creer ?? false) {
					$product->fields[$fieldKey]['arrayofkeyval'][0] = $langs->trans('ProduitRapideProduct');
				}
				if ($user->rights->service->creer ?? false) {
					$product->fields[$fieldKey]['arrayofkeyval'][1] = $langs->trans('ProduitRapideService');
				}
				$product->fields[$fieldKey]['default'] = 0;
			} else {
				if ($user->rights->service->creer ?? false) {
					$product->fields[$fieldKey]['arrayofkeyval'][1] = $langs->trans('ProduitRapideService');
				}
				if ($user->rights->produit->creer ?? false) {
					$product->fields[$fieldKey]['arrayofkeyval'][0] = $langs->trans('ProduitRapideProduct');
				}
				$product->fields[$fieldKey]['default'] = 1;
			}
		}

		$value = getInputValue($fieldKey);

		$htmlToPrint .= '<tr>';
		$htmlToPrint .= '<td>';
		$htmlToPrint .= '<span class="' . (!empty($field['notnull']) ? 'fieldrequired' : '') . '">';
		// prints label
		$htmlToPrint .= $langs->trans($field['label']);

		$htmlToPrint .= '</span>';
		$htmlToPrint .= '</td>';
		$htmlToPrint .= '<td class="valuefieldcreate">';
		// Prints files input or field input
		if ($fieldKey === 'fp_files') {
			// Form to import files
			$htmlToPrint .= '<input class="flat minwidth400 maxwidth200onsmartphone" type="file" name="userfile[]" multiple accept>';
		} else if ($fieldKey === 'fp_fk_supplier') {
			// Supplier select
			// [#21] Supplier is pre-filled on supplier order (if we don't check $object != null, there is an error in Dolibarr v13)
			if ($object != null && get_class($object) == "CommandeFournisseur") {
				$value = $object->fourn_id;
			}
			$htmlToPrint .= img_picto('', 'company', 'class="pictofixedwidth"') . $form->select_company($value, $fieldKey, 'fournisseur=1', 'SelectThirdParty');
		} else if (in_array($fieldKey, array('fp_price', 'fp_supplierprice'))) {
			// Input type number for prices
			$htmlToPrint .= '<input type="text" class="flat maxwidth75 --success maxwidthonsmartphone" name="' . $fieldKey . '" id="' . $fieldKey . '" value="' . $value . '" onkeypress="preventNonNumericalInput(event)" placeholder="0.00">';
			// HT / TTC select
			$htmlToPrint .= '<select class="flat maxwidth100" id="select_price_base_type" name="' . $fieldKey . '_type">';
			$htmlToPrint .= '<option value="HT" selected="">' . $langs->trans('HT') . '</option>';
			$htmlToPrint .= '<option value="TTC">' . $langs->trans('TTC') . '</option>';
			$htmlToPrint .= '</select>';
			$isBuyer = ($fieldKey == 'fp_price');
			$htmlToPrint .= '</td></tr><tr><td>' . $langs->trans('ProduitRapideVatRate' . ($isBuyer ? 'Seller' : 'Supplier')) . '</td><td ' . ($isBuyer ? '' : 'id="fourn_tva"') . '>';
			$htmlToPrint .= $form->load_tva(($isBuyer ? 'fp_vat_rate' : 'fp_vat_rate_supplier'), ($conf->global->PRODUITRAPIDE_TVA ?? 20), $mysoc, $mysoc, 0, 0, '', false, 1);
		} else {
			// Show input
			$htmlToPrint .= $product->showInputField($field, $fieldKey, $value, 'placeholder="' . ($field['placeholder'] ?? "") . '"', '', '', 0, 1);
		}
		// Prints TVA type for prices
		if (in_array($fieldKey, array('fp_price', 'fp_supplierprice'))) {
		}
		$htmlToPrint .= '</td>';
		$htmlToPrint .= '</tr>';
	}
	// Get default Barcode Type/Value and show it to user if input isn't enable
	if (($conf->global->PRODUITRAPIDE_DISPLAY_BARCODE ?? false) && ($conf->barcode->enabled ?? false)) {
		$barcode = getDefaultBarcodeValue();
		$barcodetype = getDefaultBarcodeType();
		// Show default barcode if it is enabled
		if ($barcode ?? false) {
			$htmlToPrint .= '<tr>';
			$htmlToPrint .= '<td colspan="2">';
			$htmlToPrint .= $langs->trans('ProduitRapideDefaultBarcodeValueWarning');
			$htmlToPrint .= ' <b>' . $barcode . '</b>';
			$htmlToPrint .= '</td>';
			$htmlToPrint .= '</tr>';
		}
		// Show default barcode type if it is enabled
		if ($barcodetype ?? false) {
			$htmlToPrint .= '<tr>';
			$htmlToPrint .= '<td colspan="2">';
			$htmlToPrint .= $langs->trans('ProduitRapideDefaultBarcodeTypeWarning');
			$htmlToPrint .= ' <b>' . getBarcodeType($barcodetype) . '</b>';
			$htmlToPrint .= '</td>';
			$htmlToPrint .= '</tr>';
		}
	}
	// [#30] Default warehouse
	if ($conf->stock->enabled ?? false) {
		$defaultWarehouse = (int) ($conf->global->MAIN_DEFAULT_WAREHOUSE ?? 0);
		$stock = new Entrepot($db);
		if ($defaultWarehouse > 0 && $stock->fetch($defaultWarehouse)) {
			$htmlToPrint .= '<input type="hidden" value="' . $defaultWarehouse . '" name="fp_default_warehouse">';
			$htmlToPrint .= '<tr>';
			$htmlToPrint .= '<td>';
			$htmlToPrint .= $langs->trans('ProduitRapideDefaultWarehouseWarning');
			$htmlToPrint .= '</td>';
			$htmlToPrint .= '<td>';
			$htmlToPrint .= ' <b>' . $stock->getNomUrl(1) . '</b>';
			$htmlToPrint .= '</td>';
			$htmlToPrint .= '</tr>';
		}
	}
	$htmlToPrint .= '<tr>';
	$htmlToPrint .= '<td></td><td id="td-buttons" align="right">';
	$htmlToPrint .= '</td>';
	$htmlToPrint .= '</tr>';
	$htmlToPrint .= '</tbody>';
	$htmlToPrint .= '</table>';
	$htmlToPrint .=  dol_get_fiche_end();
	$htmlToPrint .= '<div class="center">';
	$htmlToPrint .= '<a id="submitButtonFP" class="butAction"><span class="fa fa-cube"></span> ' . $langs->trans('Create') . '</a>';
	$htmlToPrint .= '</div>';
	$htmlToPrint .= '</form>';

	$htmlToPrint .= '</div>';

	return $htmlToPrint;
}

/**
 * Get value of field if exists
 *
 * @param string $field key of field to get value
 * @return array|float|int|string Input value
 */
function getInputValue($field)
{
	global $conf;

	// If product creation failed, get value to display it again into inputs
	$value = GETPOST($field);

	if (!$value && $field === 'fp_ref') {
		$value = getDefaultRefValue($conf->global->PRODUITRAPIDE_DEFAULT_CREATE ?? 0);
	} else if (!$value && $field === "fp_barcode_type") {
		$value = getDefaultBarcodeType();
	} else if (!$value && $field === "fp_barcode") {
		$value = getDefaultBarcodeValue();
	}

	return $value;
}

/**
 * Gets reference default value
 *
 * @param int $type Type id (0 if product, 1 if service)
 * @return mixed
 */
function getDefaultRefValue($type)
{
	require_once DOL_DOCUMENT_ROOT . '/product/class/product.class.php';

	global $db, $conf;

	if (empty($conf->global->PRODUCT_GENERATE_REF_AFTER_FORM)) {
		$module = (!empty($conf->global->PRODUCT_CODEPRODUCT_ADDON) ? $conf->global->PRODUCT_CODEPRODUCT_ADDON : 'mod_codeproduct_leopard');
		if (substr($module, 0, 16) == 'mod_codeproduct_' && substr($module, -3) == 'php') {
			$module = substr($module, 0, dol_strlen($module) - 4);
		}
		$result = dol_include_once('/core/modules/product/'.$module.'.php');
		if ($result > 0) {
			$modCodeProduct = new $module();
		}
		if (!empty($modCodeProduct->code_auto)) {
			$product = new Product($db);
			$value = $modCodeProduct->getNextValue($product, $type);
		}
	}
	return $value ?? "";
}

/**
 * Gets default barcode value
 *
 * @return null
 */
function getDefaultBarcodeValue()
{

	global $conf, $db;

	$value = null;

	// Import barcode lib
	if (!empty($conf->barcode->enabled) && !empty($conf->global->BARCODE_PRODUCT_ADDON_NUM)) {
		$module = strtolower($conf->global->BARCODE_PRODUCT_ADDON_NUM);
		$dirbarcode = array_merge(array('/core/modules/barcode/'), $conf->modules_parts['barcode']);
		foreach ($dirbarcode as $dirroot) {
			$res = dol_include_once($dirroot . $module . '.php');
			if ($res) {
				break;
			}
		}
		if ($res > 0) {
			$modBarCodeProduct = new $module();
		}
	}
	if (empty($tmpcode) && !empty($modBarCodeProduct->code_auto)) {
		$product = new Product($db);
		$value = $modBarCodeProduct->getNextValue($product, $conf->global->PRODUIT_DEFAULT_BARCODE_TYPE);
	}

	return $value;
}

/**
 * Gets default barcode type
 *
 * @return int Return 0 if there is no default barcode type or barcode id
 */
function getDefaultBarcodeType(): int
{
	global $conf;

	require_once DOL_DOCUMENT_ROOT . '/core/lib/functions.lib.php';
	if (!empty($conf->global->PRODUIT_DEFAULT_BARCODE_TYPE)) {
		$value = getDolGlobalInt("PRODUIT_DEFAULT_BARCODE_TYPE");
	} else {
		$value = 0;
	}
	return $value;
}

/**
 * Get barcode type label
 *
 * @param int $id The id of barcode type
 * @return null Returns barcode type label or null if not found
 */
function getBarcodeType($id)
{

	global $conf, $db;

	$sql = "SELECT rowid, code, libelle";
	$sql .= " FROM " . MAIN_DB_PREFIX . "c_barcode_type";
	$sql .= " WHERE rowid = " . $id;
	$sql .= " AND entity = " . $conf->entity;

	$result = $db->query($sql);
	if ($result) {
			$obj = $db->fetch_object($result);
			return $obj->libelle;
	}
	return null;
}

/**
 * GlobalCard is Dolibarr hook context of all card.php for addMoreActionsButtons hook
 *
 * @param   CommonObject    $object     The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
 * @param   Translate       $langs      The object given by dolibarr global (global $langs)
 * @return  void
 */
function buttonContextGlobalCard($object, $langs)
{
	global $user;

	if (($user->rights->produit->creer ?? false) || ($user->rights->service->creer ?? false)) {
		print '<a data-fancybox class="butAction" href="#fastproduct"><span class="fas fa-cube"></span> ' . $langs->trans("ProduitRapideQuickAdd") . '</a>';
		print generateFastProductWindow($object);
	} else {
		print '<a class="butActionRefused" href="#">' .
			$langs->trans("ProduitRapideQuickAdd") . '</a>';
	}
}


/**
 * Tells if a line is from a fast product or not
 *
 * @param CommonObjectLine $line Line
 * @return int
 */
function isFastProductLine($line)
{

	global $db;

	$fastProductTag = getFastProductTagId();

	if ($fastProductTag > 0 && !empty($line->id)) {
		$sql = "SELECT fk_product FROM " . MAIN_DB_PREFIX . $line->table_element . " WHERE rowid = " . $line->id;

		$result = $db->query($sql);
		if ($result) {
			$obj = $db->fetch_object($result);

			if (!empty($obj)) {
				$sql = "SELECT COUNT(*) as nb FROM " . MAIN_DB_PREFIX . "categorie_product WHERE fk_categorie = " . $fastProductTag . " AND fk_product = " . $obj->fk_product;

				$result = $db->query($sql);
				if ($result) {
					$obj = $db->fetch_object($result);
					$nb = $obj->nb;
					return (!empty($nb) ? 1 : 0);
				}
			}
		}
	}

	return 0;
}

/**
 * Gets fast product tag id
 *
 * @return int
 */
function getFastProductTagId()
{
	global $db, $conf;

	$sql = "SELECT rowid FROM " . MAIN_DB_PREFIX . "categorie WHERE type = 0 AND label = 'Produit Rapide' AND entity = " . $conf->entity;

	$result = $db->query($sql);
	if (!empty($result)) {
		$obj = $db->fetch_object($result);
		return $obj->rowid;
	}

	return -1;
}
